//      i2c.c
//		14.01.2013
//      
//      Copyright 2013 stefan <stefan@debian>
//      
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.

#include <stdio.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <string.h>
#include <unistd.h>
#include "i2c.h"




int _DEBUG = 0;

// 
// name: I2C_Open
// @param:
//			*fd - File descriptor
//			address - address of the slave device
//	
// @return: NONE
void I2C_Open(int *fd, char *I2CNAME, unsigned char address)
{
    if(_DEBUG)
        printf("Opening file descriptor: %s...", I2CNAME);
    *fd = open(I2CNAME, O_RDWR);
	
	//Check for fault
	if(*fd < 0){
		if(_DEBUG)
			printf("FAIL\n");			
		exit(1);
	}
	if(_DEBUG)
		printf("DONE\n");
		
	if(_DEBUG)
		printf("Opening I2C-bus...");
		
	if(ioctl(*fd, I2C_SLAVE_FORCE, address) < 0)
	{
		if(_DEBUG)
			printf("FAIL\n");			
		exit(1);		
	}
	if(_DEBUG)
		printf("DONE\n");
	
}
// 
// name: I2C_Close
// @param:
//			*fd - File descriptor
//	
// @return: NONE
void I2C_Close(int *fd){

	if(_DEBUG)
		printf ("Closing...");

	if(close(*fd) < 0)
	{
		if(_DEBUG)
			printf("FAIL\n");
		exit(1);
	}
	if(_DEBUG)
		printf("DONE\n");
}
// 
// name: I2C_Send
// @param:
//			*fd - File descriptor
//			*buffer - array of data to be sent
//			num_bytes - number of bytes to be sent 
//	
// @return: NONE
void I2C_Send(int *fd, unsigned char *buffer, unsigned char num_bytes)
{
	int count = 0;
	if(_DEBUG)
		printf("Sending...");
		
	count = write(*fd, buffer, num_bytes);
	if(count != num_bytes)
	{
		if(_DEBUG)
		{
			printf("FAIL\n");
			if(count >= 0)
				printf("%d of %d bytes send\n", count, num_bytes); 
		}
		exit(1);
	}	
	if(_DEBUG){
		printf("DONE\n");
	}
}
// 
// name: I2C_Read
// @param:
//			*fd - File descriptor
//			*buffer - array of data to be read
//			num_bytes - number of bytes to be read 
//	
// @return: NONE
void I2C_Read(int *fd, unsigned char *buffer, unsigned char num_bytes){
	
	int count = 0;
	if(_DEBUG)
		printf("Readinging...");

	count = read(*fd, buffer, num_bytes);
	if(count != num_bytes)
	{
		if(_DEBUG)
		{
			printf("FAIL\n");
			if(count >= 0)
				printf("%d of %d bytes read\n", count, num_bytes); 
		}
		exit(1);
	}
	if(_DEBUG)
		printf("DONE\n");
}
